"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __spreadArrays = (this && this.__spreadArrays) || function () {
    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
    for (var r = Array(s), k = 0, i = 0; i < il; i++)
        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
            r[k] = a[j];
    return r;
};
Object.defineProperty(exports, "__esModule", { value: true });
var GunGraphConnector_1 = require("./GunGraphConnector");
var generateMessageId_1 = require("./generateMessageId");
var GunEvent_1 = require("./GunEvent");
var reconnecting_websocket_1 = require("reconnecting-websocket");
var PING = 60000;
var WebSocketGraphConnector = /** @class */ (function (_super) {
    __extends(WebSocketGraphConnector, _super);
    function WebSocketGraphConnector(url, WS) {
        if (WS === void 0) { WS = WebSocket; }
        var _this = _super.call(this) || this;
        _this._queuedMessages = [];
        _this.events = {
            connection: new GunEvent_1.GunEvent("WS connection " + url),
            msgIn: new GunEvent_1.GunEvent("msgIn " + url),
            msgOut: new GunEvent_1.GunEvent("msgOn " + url)
        };
        _this.isConnected = false;
        _this.url = url;
        _this._requests = {};
        _this._requestsBySoul = {};
        _this._puts = {};
        _this._ws = new reconnecting_websocket_1.default(_this.url.replace(/^http/, 'ws'), [], {
            WebSocket: WS
        });
        _this._ws.addEventListener('message', _this._onReceiveSocketData.bind(_this));
        _this._ws.addEventListener('open', _this.onSocketConnect.bind(_this));
        _this._ws.addEventListener('close', _this.onSocketDisconnect.bind(_this));
        return _this;
    }
    WebSocketGraphConnector.prototype.request = function (souls) {
        var msgs = [];
        for (var i = 0; i < souls.length; i++) {
            var soul = souls[i];
            if (soul in this._requestsBySoul)
                continue;
            var msgId = generateMessageId_1.generateMessageId();
            this._requests[msgId] = soul;
            this._requestsBySoul[soul] = msgId;
            msgs.push({
                '#': msgId,
                get: {
                    '#': soul
                }
            });
        }
        this._send(msgs);
    };
    WebSocketGraphConnector.prototype.put = function (data, ackEvt) {
        var msgId = generateMessageId_1.generateMessageId();
        if (ackEvt)
            this._puts[msgId] = ackEvt;
        this._send([
            {
                '#': msgId,
                put: data
            }
        ]);
    };
    WebSocketGraphConnector.prototype.onSocketConnect = function () {
        var _this = this;
        this.isConnected = true;
        this._pingInterval = setInterval(function () { return _this._ws.send('[]'); }, PING);
        this._send(this._queuedMessages);
        this._queuedMessages = [];
        this.events.connection.trigger(true);
    };
    WebSocketGraphConnector.prototype.onSocketDisconnect = function () {
        this.isConnected = false;
        if (this._pingInterval)
            clearInterval(this._pingInterval);
        this.events.connection.trigger(false);
    };
    WebSocketGraphConnector.prototype._send = function (msgs) {
        var _a;
        var _this = this;
        if (!this.isConnected) {
            (_a = this._queuedMessages).splice.apply(_a, __spreadArrays([0, 0], msgs));
            return;
        }
        if (!msgs.length)
            return;
        if (msgs.length === 1) {
            this._ws.send(JSON.stringify(msgs[0]));
        }
        else if (msgs.length > 0) {
            this._ws.send(JSON.stringify(msgs));
        }
        msgs.forEach(function (msg) { return _this.events.msgOut.trigger(msg, _this.url); });
    };
    WebSocketGraphConnector.prototype._onReceiveMessage = function (msg) {
        var _a;
        if (!msg)
            return;
        this.events.msgIn.trigger(msg, this.url);
        var respondingTo = msg['@'];
        var putEvt = respondingTo && this._puts[respondingTo];
        if (putEvt) {
            putEvt.trigger(msg);
            return;
        }
        if (!('put' in msg))
            return;
        var requestedSoul = respondingTo && this._requests[respondingTo];
        if (msg.put) {
            this.graphData.trigger(msg.put);
        }
        else if (requestedSoul) {
            this.graphData.trigger((_a = {},
                _a[requestedSoul] = undefined,
                _a));
        }
    };
    WebSocketGraphConnector.prototype._onReceiveSocketData = function (msg) {
        var raw = msg.data || msg;
        var json = JSON.parse(raw);
        if (Array.isArray(json)) {
            json.forEach(this._onReceiveMessage.bind(this));
        }
        else {
            this._onReceiveMessage(json);
        }
    };
    return WebSocketGraphConnector;
}(GunGraphConnector_1.GunGraphConnector));
exports.WebSocketGraphConnector = WebSocketGraphConnector;
//# sourceMappingURL=WebSocketGraphConnector.js.map